==> appdynamics-4.0.0.0b4-py2-none-any.whl: appdynamics/agent/models/exitcalls.py <==
"""Model calls made from the instrumented application to an external service.

"""

from appdynamics.lib import iteritems, make_name_value_pairs
from appdynamics.agent import pb


EXIT_DB = pb.EXIT_DB
EXIT_HTTP = pb.EXIT_HTTP
EXIT_CACHE = pb.EXIT_CACHE
EXIT_QUEUE = pb.EXIT_RABBITMQ


class Backend(object):
    """A backend is the destination of an exit call.

    Parameters
    ----------
    exit_point_type : {EXIT_DB, EXIT_HTTP, EXIT_CACHE, EXIT_QUEUE}
        The type of the backend.
    identifying_properties : dict
        A dictionary of properties that uniquely identify this backend.
    name : str
        The display name of the backend.
    registered_id : int, optional
        If this backend was registered, the id given to it. The default is
        `None`.
    component_id : int, optional
        Set when this backend is registered and has been resolved to either a
        tier or an app. In the case that it has resolved to a tier in the same
        app, then this is the id of that tier. If it has resolved to a
        different app, then this is the id of that app.
    is_foreign_app : bool, optional
        True if the component_id is the id of a foreign app.

    Attributes
    ----------
    backend_type_name : str
        The name of the exit point type constant as a str (e.g., `'EXIT_DB'`).

    optional_properties : dict
        Any optional (non-identifying) properties associated with this
        backend. These are visible in the UI, but not used to identify the
        backend.

    """
    __slots__ = (
        'exit_point_type', 'identifying_properties', 'name',
        'registered_id', 'component_id', 'optional_properties',
        'is_foreign_app'
    )

    def __init__(self, exit_point_type, identifying_properties, name,
                 registered_id=None, component_id=None, is_foreign_app=False):
        super(Backend, self).__init__()

        self.exit_point_type = exit_point_type
        self.identifying_properties = identifying_properties
        self.name = name
        self.registered_id = registered_id
        self.component_id = component_id
        self.is_foreign_app = is_foreign_app

        self.optional_properties = {}

    @property
    def sorted_identifying_properties(self):
        properties = iteritems(self.identifying_properties)
        sorted_properties = sorted(properties, key=lambda p: p[0])
        return tuple(sorted_properties)

    @property
    def backend_type_name(self):
        full_name = pb.ExitPointType.Name(self.exit_point_type)
        return full_name.replace('EXIT_', '')

    def __str__(self):
        return 'Backend: exit type={0}, identifying properties={1}, name={2}'.format(
            self.exit_point_type, self.identifying_properties, self.name)


class ExitCall(object):
    """A call from the instrumented application to an external service.

    An exit call may be to a database, web application/service, or any other
    supported type of backend (such as a cache or message queue).

    Parameters
    ----------
    exit_call_id : int
        The id assigned to this exit call.
    timer : appdynamics.agent.core.timer.Timer
        The timer for keeping track of the exit call's duration.
    backend : Backend
        The backend that is the destination of this exit call.
    category : str or None
        The category this call belongs to (an arbitrary string used for
        display/categorization purposes).
    params : list or None
        The parameters associated with the operation, if any.
    caller : appdynamics.agent.models.callgraph.FrameInfo
        Information about the caller that initiated this exit call.
    operation : str, optional
        The operation that was performed as part of the exit call. It can be
        a URL, a SQL query, etc.

    Attributes
    ----------
    id : int
        The id assigned to this exit call.
    num_calls : int
        The number of times this exit call occurred during the transaction.
    num_errors : int
        The number of errors that occurred when trying perform this exit call.
    time_taken_ms : int
        The total time taken (in milliseconds) by this exit call during the
        transaction.
    min_time_taken_ms : int
        The minimum time taken (in milliseconds) by this exit call during the
        transaction. If there was only one call, this is equivalent to both
        `time_taken_ms` and `max_time_taken_ms`.
    max_time_taken_ms : int
        The maximum time taken (in milliseconds) by this exit call during the
        transaction. If there was only one call, this is equivalent to both
        `time_taken_ms` and `min_time_taken_ms`.

    """
    __slots__ = (
        'sequence_info', 'timer', 'backend', 'category',
        'operation', 'params',
        'num_calls', 'num_errors',
        'time_taken_ms', 'min_time_taken_ms', 'max_time_taken_ms',
        'caller', 'was_added', '__weakref__'
    )

    def __init__(self, sequence_info, timer, backend, category, params, caller, operation=None):
        super(ExitCall, self).__init__()
        self.sequence_info = sequence_info
        self.timer = timer

        self.backend = backend
        self.category = category
        self.operation = operation
        self.params = params

        self.num_calls = 0
        self.time_taken_ms = 0
        self.min_time_taken_ms = None
        self.max_time_taken_ms = 0

        self.num_errors = 0
        self.caller = caller
        self.was_added = False

    def add_call(self, had_error):
        time_ms = self.timer.stop()
        self.timer.reset()

        self.num_calls += 1
        self.time_taken_ms += time_ms

        if had_error:
            self.num_errors += 1

        if self.min_time_taken_ms is None or time_ms < self.min_time_taken_ms:
            self.min_time_taken_ms = time_ms

        if time_ms > self.max_time_taken_ms:
            self.max_time_taken_ms = time_ms


def make_backend_identifier_dict(backend):
    if backend.registered_id:
        identifier = {
            'type': pb.BackendIdentifier.REGISTERED,
            'registeredBackend': {
                'exitPointType': backend.exit_point_type,
                'backendID': backend.registered_id,
                'componentID': backend.component_id,
                'componentIsForeignAppID': backend.is_foreign_app,
            },
        }
    else:
        identifier = {
            'type': pb.BackendIdentifier.UNREGISTERED,
            'unregisteredBackend': {
                'exitCallInfo': {
                    'exitPointType': backend.exit_point_type,
                    'displayName': backend.name,
                    'identifyingProperties': make_name_value_pairs(backend.identifying_properties),
                },
            },
        }

    return identifier


def make_backend_metrics_dicts(exit_calls):
    backends = {}

    for call in exit_calls:
        key = (call.backend.exit_point_type, call.backend.sorted_identifying_properties, call.category)
        try:
            stats = backends[key]
            stats['timeTaken'] += call.time_taken_ms
            stats['numOfCalls'] += call.num_calls
            stats['numOfErrors'] += call.num_errors
            stats['minCallTime'] = min(stats['minCallTime'], call.min_time_taken_ms)
            stats['maxCallTime'] = max(stats['maxCallTime'], call.max_time_taken_ms)
        except:
            backends[key] = {
                'category': call.category,
                'timeTaken': call.time_taken_ms,
                'numOfCalls': call.num_calls,
                'numOfErrors': call.num_errors,
                'minCallTime': call.min_time_taken_ms,
                'maxCallTime': call.max_time_taken_ms,
                'backendIdentifier': make_backend_identifier_dict(call.backend),
            }

    return backends.values()


def make_backend_reresolve_dict(backend_id):
    try:
        reresolution = {
            'backendId': int(backend_id),
        }
        return reresolution
    except:
        return None

==> appdynamics-4.0.6.0-py2-none-any.whl: appdynamics/agent/models/exitcalls.py <==
# Copyright (c) AppDynamics, Inc., and its affiliates
# 2015
# All Rights Reserved

"""Model calls made from the instrumented application to an external service.

"""

from appdynamics.lib import iteritems, make_name_value_pairs
from appdynamics.agent import pb


EXIT_DB = pb.EXIT_DB
EXIT_HTTP = pb.EXIT_HTTP
EXIT_CACHE = pb.EXIT_CACHE
EXIT_QUEUE = pb.EXIT_RABBITMQ


class Backend(object):
    """A backend is the destination of an exit call.

    Parameters
    ----------
    exit_point_type : {EXIT_DB, EXIT_HTTP, EXIT_CACHE, EXIT_QUEUE}
        The type of the backend.
    identifying_properties : dict
        A dictionary of properties that uniquely identify this backend.
    name : str
        The display name of the backend.
    registered_id : int, optional
        If this backend was registered, the id given to it. The default is
        `None`.
    component_id : int, optional
        Set when this backend is registered and has been resolved to either a
        tier or an app. In the case that it has resolved to a tier in the same
        app, then this is the id of that tier. If it has resolved to a
        different app, then this is the id of that app.
    is_foreign_app : bool, optional
        True if the component_id is the id of a foreign app.

    Attributes
    ----------
    backend_type_name : str
        The name of the exit point type constant as a str (e.g., `'EXIT_DB'`).

    optional_properties : dict
        Any optional (non-identifying) properties associated with this
        backend. These are visible in the UI, but not used to identify the
        backend.

    """
    __slots__ = (
        'exit_point_type', 'identifying_properties', 'name',
        'registered_id', 'component_id', 'optional_properties',
        'is_foreign_app'
    )

    def __init__(self, exit_point_type, identifying_properties, name,
                 registered_id=None, component_id=None, is_foreign_app=False):
        super(Backend, self).__init__()

        self.exit_point_type = exit_point_type
        self.identifying_properties = identifying_properties
        self.name = name
        self.registered_id = registered_id
        self.component_id = component_id
        self.is_foreign_app = is_foreign_app

        self.optional_properties = {}

    @property
    def sorted_identifying_properties(self):
        properties = iteritems(self.identifying_properties)
        sorted_properties = sorted(properties, key=lambda p: p[0])
        return tuple(sorted_properties)

    @property
    def backend_type_name(self):
        full_name = pb.ExitPointType.Name(self.exit_point_type)
        return full_name.replace('EXIT_', '')

    def __str__(self):
        return 'Backend: exit type={0}, identifying properties={1}, name={2}'.format(
            self.exit_point_type, self.identifying_properties, self.name)


class ExitCall(object):
    """A call from the instrumented application to an external service.

    An exit call may be to a database, web application/service, or any other
    supported type of backend (such as a cache or message queue).

    Parameters
    ----------
    exit_call_id : int
        The id assigned to this exit call.
    timer : appdynamics.agent.core.timer.Timer
        The timer for keeping track of the exit call's duration.
    backend : Backend
        The backend that is the destination of this exit call.
    category : str or None
        The category this call belongs to (an arbitrary string used for
        display/categorization purposes).
    params : list or None
        The parameters associated with the operation, if any.
    caller : appdynamics.agent.models.callgraph.FrameInfo
        Information about the caller that initiated this exit call.
    operation : str, optional
        The operation that was performed as part of the exit call. It can be
        a URL, a SQL query, etc.

    Attributes
    ----------
    id : int
        The id assigned to this exit call.
    num_calls : int
        The number of times this exit call occurred during the transaction.
    num_errors : int
        The number of errors that occurred when trying perform this exit call.
    time_taken_ms : int
        The total time taken (in milliseconds) by this exit call during the
        transaction.
    min_time_taken_ms : int
        The minimum time taken (in milliseconds) by this exit call during the
        transaction. If there was only one call, this is equivalent to both
        `time_taken_ms` and `max_time_taken_ms`.
    max_time_taken_ms : int
        The maximum time taken (in milliseconds) by this exit call during the
        transaction. If there was only one call, this is equivalent to both
        `time_taken_ms` and `min_time_taken_ms`.

    """
    __slots__ = (
        'sequence_info', 'timer', 'backend', 'category',
        'operation', 'params',
        'num_calls', 'num_errors',
        'time_taken_ms', 'min_time_taken_ms', 'max_time_taken_ms',
        'caller', 'was_added', '__weakref__', 'was_assigned'
    )

    def __init__(self, sequence_info, timer, backend, category, params, caller, operation=None):
        super(ExitCall, self).__init__()
        self.sequence_info = sequence_info
        self.timer = timer

        self.backend = backend
        self.category = category
        self.operation = operation
        self.params = params

        self.num_calls = 0
        self.time_taken_ms = 0
        self.min_time_taken_ms = None
        self.max_time_taken_ms = 0

        self.num_errors = 0
        self.caller = caller
        self.was_added = False
        self.was_assigned = False

    def add_call(self, had_error):
        time_ms = self.timer.stop()
        self.timer.reset()

        self.num_calls += 1
        self.time_taken_ms += time_ms

        if had_error:
            self.num_errors += 1

        if self.min_time_taken_ms is None or time_ms < self.min_time_taken_ms:
            self.min_time_taken_ms = time_ms

        if time_ms > self.max_time_taken_ms:
            self.max_time_taken_ms = time_ms


def make_backend_identifier_dict(backend):
    if backend.registered_id:
        identifier = {
            'type': pb.BackendIdentifier.REGISTERED,
            'registeredBackend': {
                'exitPointType': backend.exit_point_type,
                'backendID': backend.registered_id,
                'componentID': backend.component_id,
                'componentIsForeignAppID': backend.is_foreign_app,
            },
        }
    else:
        identifier = {
            'type': pb.BackendIdentifier.UNREGISTERED,
            'unregisteredBackend': {
                'exitCallInfo': {
                    'exitPointType': backend.exit_point_type,
                    'displayName': backend.name,
                    'identifyingProperties': make_name_value_pairs(backend.identifying_properties),
                },
            },
        }

    return identifier


def make_backend_metrics_dicts(exit_calls):
    backends = {}

    for call in exit_calls:
        key = (call.backend.exit_point_type, call.backend.sorted_identifying_properties, call.category)
        try:
            stats = backends[key]
            stats['timeTaken'] += call.time_taken_ms
            stats['numOfCalls'] += call.num_calls
            stats['numOfErrors'] += call.num_errors
            stats['minCallTime'] = min(stats['minCallTime'], call.min_time_taken_ms)
            stats['maxCallTime'] = max(stats['maxCallTime'], call.max_time_taken_ms)
        except:
            backends[key] = {
                'category': call.category,
                'timeTaken': call.time_taken_ms,
                'numOfCalls': call.num_calls,
                'numOfErrors': call.num_errors,
                'minCallTime': call.min_time_taken_ms,
                'maxCallTime': call.max_time_taken_ms,
                'backendIdentifier': make_backend_identifier_dict(call.backend),
            }

    return backends.values()


def make_backend_reresolve_dict(backend_id):
    try:
        reresolution = {
            'backendId': int(backend_id),
        }
        return reresolution
    except:
        return None
